% source2raw Organization of source data into raw folder.
% 
% This script reads all .vol files in data folders and organizes them
% in a BIDS-like folder structure.
% It is only designed to handle in-house raw data so it is not intended to
% be directly used by others.
%
% 2021, Mondragon Unibertsitatea, Biomedical Engineering Department
% -------------------------------------------------------------------------


% -------------------------------------------------------------------------
% 1. Define data folders and initiate participants table
% -------------------------------------------------------------------------

close all;clc;clearvars;

addpath(genpath('../../utils/'));

% ----------------------- Folder Structure -----------------------------
inputPath = '../../../data/source';

% Create output folder if necessary. Delete if it was already created.
outputPath = '../../../data/raw';
if exist(outputPath,'dir')
    rmdir(outputPath,'s'); % Force to remove even if it contains files
end
mkdir(outputPath);
% ----------------------------------------------------------------------


% ------------------- Visual Inspection of results ----------------------
savePng = true;

if savePng
    mkdir([outputPath '/0_png/']);
end
% ----------------------------------------------------------------------


% ------------------- Data to process ----------------------------------
% Generate subject table with basic demographics and acquired scans
varNames = {'subjectId','group','age','sex',...
    'baseline_oct_macularRaster_OD','baseline_oct_macularRaster_OS',...
    'baseline_oct_macularStar_OD','baseline_oct_macularStar_OS',...
    'baseline_fundus_macularRaster_OD','baseline_fundus_macularRaster_OS',...
    'baseline_fundus_macularStar_OD','baseline_fundus_macularStar_OS'};

nVars = length(varNames); 

participants  = cell2table(repmat({'Init'},1,nVars),'VariableNames',varNames);


% Paths to all data folders
dataFolders = {[inputPath '/01_OCT_MJFF/01_MJFF_year_0/MJFF_year_0_OCT/VOL_macula/'],...
    [inputPath '/01_OCT_MJFF/01_MJFF_year_0/MJFF_year_0_OCT/VOL_macula_star/'],...
    [inputPath '/02_OCT_EITB/EITB_year_0/EITB_year_0_OCT/VOL_macula_raster/'],...
    [inputPath '/02_OCT_EITB/EITB_year_0/EITB_year_0_OCT/VOL_macula_star/'],...
    [inputPath '/03_OCT_VIRTUAL_GAMES/VIRTUAL_GAMES_year_0_OCT/VOL_macula_raster/'],...
    [inputPath '/03_OCT_VIRTUAL_GAMES/VIRTUAL_GAMES_year_0_OCT/VOL_macula_star/']};

% Set session of each datafolder
session = {'baseline','baseline','baseline',...
    'baseline','baseline','baseline',...
    'baseline','baseline','baseline',...
    'baseline'};

% modality = 'oct'; %optionally add fundus to read also fundus images (messy)
 
% Scan Type of each data folder
scanType = {'macularRaster','macularStar',... % MJFF Baseline
    'macularRaster','macularStar',... % EITB Telemaratoia
    'macularRaster','macularStar'}; %  Virtual Games

% ------------------------------------------------------------------------

% ------------------------------------------------------------------------
% 2. Process all data folders
% ------------------------------------------------------------------------       

% list to check duplicated scan names
scanList = {};
n=1;        

nFolders = length(dataFolders);

for iFolder = 1:nFolders
    % Check folder
    if ~isfolder(dataFolders{iFolder})
        error(['Folder does not exist:' dataFolders{iFolder}]);
    end

    % Get Scans in folder
    folderData = dir(dataFolders{iFolder}); % get data in folder 
    filesInFolder = vertcat({folderData.name})'; % store file names
    
    % Get only .vol and .tif files
    isVol = contains(filesInFolder,'.vol');
    scansInFolder = filesInFolder(isVol);    

    nScan = length(scansInFolder);

    if nScan ==0
        error('No scans found');        
    end
    
    disp([dataFolders{iFolder} '   ' num2str(nScan) ' scans found']);

    
    for iScan=1:nScan

        % Get subjectId of the scan
        scan = scansInFolder{iScan}; % get scan name
        ind = find(scan=='_'); % find underscore
        subjectId = scan(1:ind-1); % get subjectId

        % Check if scan is duplicated
        if sum(strcmp(scanList,scan))>0
            warning(['Duplicated scan:' scan]);
        end
        scanList{n} = scan;
        n = n+1;
        
        % Check if subject is already in the table
        if size(participants,1) == 1 % first subject (only initial row in table) 
            newSubject = true;
            subjectInd = 2;
        else
            % Get subjectIds in table
            subjectsInTable = participants.subjectId;
            % Try to find our subject
            subjectInd = find(strcmp(subjectsInTable,subjectId));    
            if isempty(subjectInd)
                newSubject = true;
                subjectInd = size(participants,1)+1;
            else
                newSubject = false;
            end       
        end
        
        % Create row and folder structure for new subject
        if newSubject
            
            % Create new row
            newRow = cell2table([{subjectId} {'group'} {0} {'sex'} repmat({'No'},1,nVars-4)],...
                'VariableNames',varNames);

            % Append row to the table
            participants = [participants;newRow];            
        end

        % Read .vol (get BScans,fundus(slo) and header data)
        
%         option = 'visu'; % plot fundus + B-Scans
%         option = 'visuseg'; % plot fundus +BScans + ILM/RPE surfaces
        option = 'nodisp';
        [volHeader, BScanHeader, fundus, BScans] = openVolFast_edit([dataFolders{iFolder} scan], option);    
        
        % Merge BScanHeader and volHeader into a single header
        header = volHeader;
        f = fields(BScanHeader);
        for iField = 1:length(f)
            header.(f{iField}) = BScanHeader.(f{iField});
        end
        
        % Get Eye (remove trailling blank spaces)
        eye = header.Eye(1:2);        

        % Save OCT in .mat files
        octFilePath = [outputPath '/' subjectId '/' session{iFolder} '/oct/' eye '/'];
        octFileName = [subjectId '_' session{iFolder} '_oct_' scanType{iFolder} '_' eye];
        if ~exist(octFilePath,'dir')
            mkdir(octFilePath);
        end
        save([octFilePath octFileName '.mat'],'header','BScans');
        
        % Save fundus in .mat files
        fundusFilePath = [outputPath '/' subjectId '/' session{iFolder} '/fundus/' eye '/'];        
        fundusFileName = [subjectId '_' session{iFolder} '_fundus_' scanType{iFolder} '_' eye];
        if ~exist(fundusFilePath,'dir')
            mkdir(fundusFilePath);
        end
        save([fundusFilePath fundusFileName '.mat'],'header','fundus');
        
                
        % Update table
        participants.([session{iFolder} '_oct_' scanType{iFolder} '_' eye])(subjectInd) = {'Yes'};
        participants.([session{iFolder} '_fundus_' scanType{iFolder} '_' eye])(subjectInd) = {'Yes'};
        
        % -------------------- Visual Inspection (fundus only) ----------
        if savePng
            imwrite(fundus,[outputPath '/0_png/' octFileName '.tif']);
        end
        % ---------------------------------------------------------------
        
%         disp(scan);        
    end
end

% delete init row
initialRow = find(strcmp(participants.subjectId,'Init'));
participants(initialRow,:) = [];

% Save updated participants table
writetable(participants,[outputPath '/participants.tsv'],'delimiter','\t','FileType','text');
       