% preprocessings Basic preprocessing of the images.
%
% This script preprocesses raw data:
% 1- Read boundary segmentation (ILM and BM)
% 2- Remove outliers
% 3. Get A-Scan coordinates in pixel space (fundus space)
%
% 2021, Mondragon Unibertsitatea, Biomedical Engineering Department
% -------------------------------------------------------------------------

clc;clearvars;close all;

addpath(genpath('../utils'));

% ---------------------- Folder Structure -------------------------------
inputPath = '../../data/raw'; % Folder from which read data
outputPath =  '../../data/derivatives/1_preprocessing';

% Create output folder if necessary. Delete if it was already created.
if exist(outputPath,'dir')
%     rmdir(outputPath,'s'); % force to remove even if it contains files
end
mkdir(outputPath);
% -----------------------------------------------------------------------

% ------------------- Visual check of results --------------------------
savePng = true;
if savePng
    mkdir([outputPath '/0_png']);
    visible = 'off';
end
%-----------------------------------------------------------------------

% Load subjects list 
opts = detectImportOptions([inputPath '/subjects.tsv'],'FileType','text');
subjects = readtable([inputPath '/subjects.tsv'],opts);
nSubject = size(subjects,1);

% Choose one eye per subject
eyes = chooseEyes(subjects);

scanTypeList = {'raster','star'};

for iSubject = 1:nSubject
    
    for iType=1:2
        % Get Scan type
        scanType = scanTypeList{iType};
        
        % Get subject and eye
        subject = subjects.subject{iSubject};
        eye = eyes{iSubject};

        % Define input folder/file
        filePath = [inputPath '/' subject '/' eye];
        fileName = [subject '_' eye '_' scanType '.mat'];

        % Load data (header with segmentations)
        header = parload([filePath '/' fileName],'header');

        % Set out of range values to nan
        th = 1e4;
        header.ILM(abs(header.ILM) > th) = nan;
        header.BM(abs(header.BM) > th) = nan;

        % Get ILM and BM boundaries in mm
        ILM_0 = (double(header.SizeZ) - double(header.ILM))*double(header.ScaleZ);
        BM_0 = (double(header.SizeZ) - double(header.BM))*double(header.ScaleZ);

        % Get coordinates
        [xAScan_0,yAScan_0] = getAScanCoordinates(header,scanType);

        % Center Coordinates (by default centred at fundus)
        [xAScan,yAScan,offset,ILM,BM] = centreCoordinates(xAScan_0,yAScan_0,scanType,ILM_0,BM_0);

        % Flip if OS
        if strcmp(eye,'OS')
           xAScan = -xAScan; 
        end                                                         

        % Define new folder and file name
        newPath = [outputPath '/' subject '/' eye];
        newFile = [subject '_' eye '_' scanType '_preproc'];

        mkdir(newPath);
        parsave([newPath '/' newFile '.mat'],xAScan,yAScan,ILM,BM);          
        % --------------------------------------------------------------------


        % --------------------- Visual Inspection ----------------------------
        if savePng
            f = figure('visible',visible);
            clf(f);

            subplot(121);
            surf(xAScan_0,yAScan_0,ILM_0 - BM_0,'EdgeColor','none');
            view(0,90);title('acquired');

            subplot(122);
            surf(xAScan,yAScan,ILM-BM,'EdgeColor','none');
            view(0,90);title('centered coords');

            sgtitle(newFile,'Interpreter','none');

            saveas(f,[outputPath '/0_png/' newFile],'png');
            close(f);
        end
        % --------------------------------------------------------------------

        disp(newFile);
    end
end
