% pit_morphology - Computation of morphological foveal pit metrics
%
% This script processes the data:
% 1- Adjust mathematical model or smooth the data
% 2- Estimate morphological parameters
%
% 2021, Mondragon Unibertsitatea, Biomedical Engineering Department
% -------------------------------------------------------------------------

clearvars;clc;close all;

addpath(genpath('../utils'));

% -------------- Loaded thickness characteristics ------------------------
% Interpolation method
interpMethod = 'cubic';
% Align image centre
alignMethod = 'smooth_min';
% Resample A-Scans to another pattern
Grid.type = 'star';
Grid.maxD = 2;
Grid.step = 0.02;
Grid.nAngles = 24;

% --------------------------- data folders -------------------------------
rawPath = '../../data/raw';
% Set input path
inputPath = ['../../data/derivatives/2_thickness/' alignMethod '_' ...
    Grid.type '_' num2str(Grid.maxD) '_' num2str(Grid.step) '_' num2str(Grid.nAngles)];

% Create folder of output
outputPath = '../../data/derivatives/3_morphology';
if exist(outputPath,'dir')
%     rmdir(outputPath,'s');
end
mkdir(outputPath);  

savePng = true;
visible = 'off';

% ------------------------------ subjects --------------------------------
% Load subjects list 
opts = detectImportOptions([rawPath '/subjects.tsv'],'FileType','text');
subjects = readtable([rawPath '/subjects.tsv'],opts);
nSubject = size(subjects,1);

% Choose one eye per subject
eyes = chooseEyes(subjects);

scanTypeList = {'raster','star'};

% -------------- Pit mnodelling characteristics --------------------------
% Modelled Layer
layerName = 'TRT';

% Pit Models
pitModelList = horzcat({'none','Scheibe','Ding','Yadav','Breher','Dubis','Liu'},...
    combineCell('movAvg_',1:2:60),...
    combineCell('loess_',1:50));

pitModelList = combineCell('movAvg_',5:5:60);

nPitModel = length(pitModelList);

% -------------------------------------------------------------------
    
for iModel = 1:nPitModel
    
    % Get pit model
    pitModel = pitModelList{iModel};    
    
    modelPath = [outputPath '/' pitModel];
    
    if ~exist(modelPath,'dir')
        mkdir(modelPath);    
    end
    
    if savePng
        mkdir([outputPath '/' pitModel '/0_png']);
    end
        
    parfor iSubject = 1:nSubject
        
        for iType=1:2
            % Get scanType
            scanType = scanTypeList{iType};

            % Get scan characteristics
            subject = subjects.subject{iSubject};
            eye = eyes{iSubject};

            searchPath = [inputPath '/' subject '/' eye];
            searchFile = [subject '_' eye '_' scanType '_thickness.mat'];

            % load oct data
            [xAScan,yAScan,TRT] = parload([searchPath '/' searchFile],'xAScan','yAScan','TRT');

            % Transform cartesian to radial 
            [theta,rho] = cart2pol(xAScan,yAScan);

            % Model 
            [TRT_fit,fitCoeff] = fitPitModel(theta,rho,TRT,pitModel);  

            % Get morphological parameters
            Paramorf = getMorphParams(rho,TRT_fit);
             
            % Define new folder and file
            newPath = [modelPath '/' subject '/' eye];        
            newFile = [subject '_' eye '_' scanType '_morphology.mat'];

            % Create new folder and save file
            if ~exist(newPath,'dir')
                mkdir(newPath);   
            end

            parsave([newPath '/' newFile],TRT_fit,Paramorf);       

            % Visual inspection of results
            if savePng
                f = figure('visible',visible);
                clf(f);
                
                subplot(121);
                plot(rho',TRT','k');grid on;title('Raw');
                ylimits = get(gca,'YLim');
                
                subplot(122);
                plot(rho',TRT_fit','b');grid on;title(pitModel,'Interpreter','none');
                ylim(ylimits);  
                
                sgtitle(newFile,'Interpreter','none');
                
                saveas(f,[modelPath '/0_png/' newFile],'png');
                close(f);
            end
            
            disp(newFile);
        end

    end
end
