function [X,Y] = getAScanCoordinates(header,scanType)
% getAscanCoordinates - Compute A-Scan coordinates from .vol data
%
% [X,Y] = getAScanCoordinates(header,scanType)
%
% Input arguments:
%   header: struct with .vol header
%   scanType: acquisition type (raster,star,onh)
%
% Output arguments:
%   X,Y: matrixes with cartesisn A-Scan coordinates
%
%  2021, Mondragon Unibertsitatea, Biomedical Engineering Department


% Global properties of fundus image 
nX = double(header.SizeXSlo);
nY = double(header.SizeYSlo);

scaleXSlo = header.ScaleXSlo;
scaleYSlo = header.ScaleYSlo;
      
switch scanType

    case 'raster'
        startX = header.StartX;
        startY = header.StartY;

        endX = header.EndX;
        endY = header.EndY;

        % Revert Y Axis
        startY = -startY;
        endY = -endY;

        % Trasnform coordinates
        % Set coordinate origin at the center of the image and revert Y axis
        xOffset = scaleXSlo*(nX-1)/2;
        yOffset = -scaleYSlo*(nY-1)/2;

        startX = startX - xOffset;
        startY = startY - yOffset;
        endX = endX -xOffset;
        endY = endY -yOffset;

        nAScans = header.NumAScans;
        nBScans = header.NumBScans;

        X = nan(nBScans,nAScans);
        Y = nan(nBScans,nAScans);
        for iBScan = 1:nBScans
            X(iBScan,:) = linspace(startX(iBScan),endX(iBScan),nAScans);
            Y(iBScan,:) = linspace(startY(iBScan),endY(iBScan),nAScans);
        end
    case 'star'
        startX = header.StartX;
        startY = header.StartY;

        endX = header.EndX;
        endY = header.EndY;

         % Revert Y Axis
        startY = -startY;
        endY = -endY;

        % Trasnform coordinates
        % Set coordinate origin at the center of the image and revert Y axis
        xOffset = scaleXSlo*(nX-1)/2;
        yOffset = -scaleYSlo*(nY-1)/2;
        startX = startX - xOffset;
        startY = startY - yOffset;
        endX = endX -xOffset;
        endY = endY -yOffset;

        nAScans = header.NumAScans;
        nBScans = header.NumBScans;

        X = nan(nBScans,nAScans);
        Y = nan(nBScans,nAScans);
        for iBScan = 1:nBScans
            % Get star angle
            theta = atan((endY(iBScan)-startY(iBScan))/(endX(iBScan)-startX(iBScan)));
            if theta <0 & endY(iBScan)> startY(iBScan)
                theta = theta + pi;
            end
            % Start End distance
            dStartEnd = sqrt((endX(iBScan)-startX(iBScan))^2+(endY(iBScan)-startY(iBScan))^2);
            % Compute radial distance assuming start as reference
            r = linspace(0,dStartEnd,nAScans);
            % Trasnform to cartesian
            [Xline,Yline] = pol2cart(theta,r);
            % Translate to its real place
            X(iBScan,:) = Xline + startX(iBScan);
            Y(iBScan,:) = Yline + startY(iBScan);
        end
end
end




