function  [TRT_fit,fitCoeff] = modelLiu(rho,TRT)
% modelLiu - fit the foveal pit model presented in Liu et al. ,2016 
%
% [TRT_fit,fitCoeff] = modelLiu(rho,TRT)
%
% Input arguments:
%   rho: matrix with rho coordinates
%   TRT: matrix with TRT points (each row is an angular direction)
%
% Output arguments:
%   TRT_fit: fitted TRT values
%   fitCoeff: estimated model coefficients
%
%  2021, Mondragon Unibertsitatea, Biomedical Engineering Department


[nAngles, nPoints] = size(TRT);

TRT_fit = nan(nAngles,nPoints);

nBScans = nAngles/2;

fitCoeff = nan(nBScans,6);

% Fitting configuration
x0 = [80 0 320 0 0 0.5];
lower = [10 -10 200 0 -1 0.1];
upper = [200 10 450 5 1 1];


fitfun = fittype('equationLiu(x,a,f,g,lambda,mu,sigma)');

options = fitoptions('StartPoint',x0,'Method','NonlinearLeastSquares',...
    'Lower',lower,'Upper',upper,'TolFun',1e-6,'TolX',1e-6,'MaxIter',1000,'Display','off');
   
for n=1:nBScans
    
    % Reconstruct the BScan
    x = [-fliplr(rho(n+nBScans,:)) rho(n, 2:end)];
    y = [fliplr(TRT(n+nBScans,:)) TRT(n, 2:end)]; % Dont get center two times
    
    % Fit the model (in um)
    fitted = fit(x',1e3*y',fitfun,options);
    
    % Reconstruct fitted curve (back to mm)
    yFit = 1e-3*fitted(x)';
    
    % From fitted B-Scan to radial again
    TRT_fit(n,:) = yFit((end-nPoints+1):end);% Right part of the B-Scan
    TRT_fit(n+nBScans,:) = yFit(nPoints:-1:1);
    
    % Store Coefficients
    fitCoeff(n,:) = [fitted.a fitted.f fitted.g fitted.lambda fitted.mu fitted.sigma];
end

