function [X_grid,Y_grid,TRT] = resampleAScan(X,Y,TRT,interpMethod,ResampleGrid)
% resampleAScan - resample A-Scan pattern to a different grid
%
% [X_grid,Y_grid,TRT] = resampleAScan(X,Y,TRT,interpMethod,ResampleGrid)
%
% Input arguments:
%   X,Y: matrixes with cartesian coordinates of A-Scan points
%   interpMethod: interpolation method to be used
%   ResampleGrid: struct specifying the output grid (regular or star)
%
% Output arguments:
%   X_grid,YT_grid: matrixes with cartesian coordinates of the resampling
%   grid
%   TRT: TRT values interpolated on the new grid
%
%  2021, Mondragon Unibertsitatea, Biomedical Engineering Department


%  Generate the resampling grid in both cartesian and polar coordinates
switch ResampleGrid.type
    
   case 'regular'
        step = ResampleGrid.step;
        maxD = ResampleGrid.maxD;
        [X_grid,Y_grid] = meshgrid(-maxD:step:maxD,-maxD:step:maxD);  
        
    case 'star'
        % Get params of grid
        nAngles = ResampleGrid.nAngles;
        step = ResampleGrid.step;
        maxD = ResampleGrid.maxD;
        
        % Define Grid
        Rho_grid = 0:step:maxD;
        Theta_grid = linspace(0,2*pi,nAngles+1);
        Theta_grid(end) = [];
        Rho_grid = repmat(Rho_grid,nAngles,1);
        Theta_grid = repmat(Theta_grid',1,size(Rho_grid,2));
        [X_grid,Y_grid] = pol2cart(Theta_grid,Rho_grid);   
end

% Interpolate over new grid

if isnumeric(TRT) % One numeric matrix 
    % Interpolation
    maskNum = ~isnan(TRT); % mask of not nan     
    
    % Interpolation (griddata does not extrapolate)    
    TRT = reshape(griddata(X(maskNum),Y(maskNum),TRT(maskNum),...
        X_grid(:),Y_grid(:),interpMethod),size(X_grid));
    
    % EXTRAPOLATION for remaining NaNs (for outside sampled region)
    % Caution not to extrapolate too far from the sampled region as
    % the quality might be very poor
    % scatteredInterpolant does not support cubic so it is linear           
    maskNum = ~isnan(TRT); % mask of not nan                
    if sum(~maskNum(:))>0
        warning('Extrapolation necessary');
        interpol = scatteredInterpolant(X_grid(maskNum),Y_grid(maskNum),TRT(maskNum),'linear');
        TRT = reshape(interpol(X_grid(:),Y_grid(:)),size(X_grid));                 
    end
else
    error('Unsupported data type');
end
